# Define Clocks
create_clock -period 8   -name master_clock1  [get_ports {master_clock1}]
create_clock -period 8   -name clk_125mhz     [get_ports {clk_125mhz}]
create_clock -period 8   -name L1_rx_clk      [get_ports {L1_rx_clk}]
create_clock -period 8   -name L2_rx_clk      [get_ports {L2_rx_clk}]

create_clock -period 4   -name virtual_clk
# added 180 degree phase shift. ( This phase shift is done through samplingclock and the ADC chips )
create_clock -period 4   -name adc_clk0       [get_ports {adc_clk[0]}]
create_clock -period 4   -name adc_clk1       [get_ports {adc_clk[1]}]
create_clock -period 4   -name adc_clk2       [get_ports {adc_clk[2]}]
create_clock -period 4   -name adc_clk3       [get_ports {adc_clk[3]}]
#create_clock -period 4   -name adc_clk0   -waveform {2 4}    [get_ports {adc_clk[0]}]
#create_clock -period 4   -name adc_clk1   -waveform {2 4}    [get_ports {adc_clk[1]}]
#create_clock -period 4   -name adc_clk2   -waveform {2 4}    [get_ports {adc_clk[2]}]
#create_clock -period 4   -name adc_clk3   -waveform {2 4}    [get_ports {adc_clk[3]}]

create_generated_clock -name shifted_adc_clk0 -source [get_ports {adc_clk[0]}] -phase 255 [get_nets {inst7|inst0|inst25|altpll_component|_clk0}]
create_generated_clock -name shifted_adc_clk1 -source [get_ports {adc_clk[1]}] -phase 255 [get_nets {inst7|inst1|inst25|altpll_component|_clk0}]
create_generated_clock -name shifted_adc_clk2 -source [get_ports {adc_clk[2]}] -phase 255 [get_nets {inst7|inst2|inst25|altpll_component|_clk0}]
create_generated_clock -name shifted_adc_clk3 -source [get_ports {adc_clk[3]}] -phase 255 [get_nets {inst7|inst3|inst25|altpll_component|_clk0}]


# Use +/- 200ps skew and center clock. This assumes a setup time of 3.8ns and a hold time of 3.8ns
# max_value = 8ns-3.8ns = 4.2ns   :   min_value = 3.8ns
set_input_delay -clock { clk_125mhz } -max   4.2 [get_ports {L1A LIVE}]
set_input_delay -clock { clk_125mhz } -min   3.8 [get_ports {L1A LIVE}] -add_delay

# Use +/- 200ps skew and center clock. This assumes a setup time of 3.8ns and a hold time of 3.8ns
# The 2.5Gbps TLK3101 RXD specs setup time of 3.0ns and hold time of 3.0ns
# max_value = 8ns-3.8ns = 4.2ns   :   min_value = 3.8ns
set_input_delay -clock { L1_rx_clk }  -max   4.2   [get_ports {L1_rx_data[*] L1_rx_error L1_rx_enable}]
set_input_delay -clock { L1_rx_clk }  -min   3.8   [get_ports {L1_rx_data[*] L1_rx_error L1_rx_enable}] -add_delay

# Use +/- 200ps skew and center clock. This assumes a setup time of 3.8ns and a hold time of 3.8ns
# The 3.125Gbps TLK3101 RXD specs setup time of 2.5ns and hold time of 2.5ns
# max_value = 6.4ns-3.0ns = 3.4ns   :   min_value = 3.0ns
set_input_delay -clock { L2_rx_clk }  -max   4.2 [get_ports {L2_rx_data[*] L2_rx_error L2_rx_enable}]
set_input_delay -clock { L2_rx_clk }  -min   3.8 [get_ports {L2_rx_data[*] L2_rx_error L2_rx_enable}] -add_delay

# Use 100ps for clock skew. ADS5463 has Tskew = -350ps and +650ps.
# Use Input minimum delay value = min trace delay for data + tCO min of external device � max trace delay for clock
#     Input minimum delay value = -100ps - 350ps = -450ps
#
# Use Input maximum delay value = maximum trace delay for data + tCO of external device � minimum trace delay for clock
#     Input maximum delay value = 100ps + 650ps = 750ps
#
# Need to add margin +/-300ps   min=-750ps max=1050ps
set_input_delay -clock { virtual_clk }             -max  .70    [get_ports adc[*]]
set_input_delay -clock { virtual_clk }             -min -.30    [get_ports adc[*]]
set_input_delay -clock { virtual_clk } -clock_fall -max  .70    [get_ports adc[*]]    -add_delay
set_input_delay -clock { virtual_clk } -clock_fall -min -.30    [get_ports adc[*]]    -add_delay
set_input_delay -clock { virtual_clk }             -max  .70    [get_ports adc_or[*]]
set_input_delay -clock { virtual_clk }             -min -.30    [get_ports adc_or[*]]
set_input_delay -clock { virtual_clk } -clock_fall -max  .70    [get_ports adc_or[*]] -add_delay
set_input_delay -clock { virtual_clk } -clock_fall -min -.30    [get_ports adc_or[*]] -add_delay

#
#  Ignore timing for clk_sync_mode    shifted clocks to main 125mhz clock
#
set_false_path -setup -fall_from [get_clocks virtual_clk] -rise_to [get_clocks shifted_adc_clk0]
set_false_path -setup -rise_from [get_clocks virtual_clk] -fall_to [get_clocks shifted_adc_clk0]
set_false_path -hold  -rise_from [get_clocks virtual_clk] -rise_to [get_clocks shifted_adc_clk0]
set_false_path -hold  -fall_from [get_clocks virtual_clk] -fall_to [get_clocks shifted_adc_clk0]
set_false_path -setup -fall_from [get_clocks virtual_clk] -rise_to [get_clocks shifted_adc_clk1]
set_false_path -setup -rise_from [get_clocks virtual_clk] -fall_to [get_clocks shifted_adc_clk1]
set_false_path -hold  -rise_from [get_clocks virtual_clk] -rise_to [get_clocks shifted_adc_clk1]
set_false_path -hold  -fall_from [get_clocks virtual_clk] -fall_to [get_clocks shifted_adc_clk1]
set_false_path -setup -fall_from [get_clocks virtual_clk] -rise_to [get_clocks shifted_adc_clk2]
set_false_path -setup -rise_from [get_clocks virtual_clk] -fall_to [get_clocks shifted_adc_clk2]
set_false_path -hold  -rise_from [get_clocks virtual_clk] -rise_to [get_clocks shifted_adc_clk2]
set_false_path -hold  -fall_from [get_clocks virtual_clk] -fall_to [get_clocks shifted_adc_clk2]
set_false_path -setup -fall_from [get_clocks virtual_clk] -rise_to [get_clocks shifted_adc_clk3]
set_false_path -setup -rise_from [get_clocks virtual_clk] -fall_to [get_clocks shifted_adc_clk3]
set_false_path -hold  -rise_from [get_clocks virtual_clk] -rise_to [get_clocks shifted_adc_clk3]
set_false_path -hold  -fall_from [get_clocks virtual_clk] -fall_to [get_clocks shifted_adc_clk3]

# out of range flag is asychronously sampled
set_false_path -from [get_registers *500*inst15*] -to [get_registers *500*inst16*]

#Clock Generation for PLL clocks 
create_generated_clock -name pll_125mhz     -source [get_ports {clk_125mhz}] -divide_by 1 -duty_cycle 50 -phase 0      [get_nets {inst12|altpll_component|_clk1}]
create_generated_clock -name pll_125mhz_inv -source [get_ports {clk_125mhz}] -divide_by 1 -duty_cycle 50 -phase 180    [get_nets {inst12|altpll_component|_clk3}]
create_generated_clock -name pll_156mhz     -source [get_ports {clk_125mhz}] -divide_by 1 -duty_cycle 50 -phase 0      [get_nets {inst11|altpll_component|_clk0}]
create_generated_clock -name pll_156mhz_inv -source [get_ports {clk_125mhz}] -divide_by 1 -duty_cycle 50 -phase 255    [get_nets {inst11|altpll_component|_clk2}]
create_generated_clock -name sample_clock   -source [get_ports {clk_125mhz}] -divide_by 1 -duty_cycle 50 -phase 72     [get_nets {inst9|altpll_component|_clk4}]
create_generated_clock -name samplingclock  -source [get_nets {inst9|altpll_component|_clk4}]  [get_ports {samplingclock}]
create_generated_clock -name L1_clk         -source [get_nets {inst12|altpll_component|_clk3}] [get_ports {L1_clk}]
create_generated_clock -name L2_clk         -source [get_nets {inst11|altpll_component|_clk2}] [get_ports {L2_clk}]


# Clock Uncertainty
set_clock_uncertainty -to { pll_125mhz } -setup 0.10
set_clock_uncertainty -to { pll_125mhz } -hold  0.10
set_clock_uncertainty -to { pll_156mhz } -setup 0.20
set_clock_uncertainty -to { pll_156mhz } -hold  0.20

# Soft error test Function (input from FPGA)
set_input_delay -clock { pll_125mhz } -max  4.0  [get_ports {read_crc_error}]
#set_input_delay -clock { pll_125mhz } -max   4.0 [get_ports {read_crc_error}]
set_input_delay -clock { pll_125mhz } -min   1.0 [get_ports {read_crc_error}]

#Output delays for 2.5Gbps Optical Links
# Use +/- 200ps skew and center clock. This assumes a setup time of 3.8ns and a hold time of 3.8ns
# The 2.5Gbps TLK3101 TXD specs setup time of 1.5ns and hold time of 0.4ns
# max_value = 3.8ns   :   min_value = -3.8ns
set_output_delay -clock { L1_clk } -max  3.8 [get_ports {L1_data*  L1_enable L1_error }]
set_output_delay -clock { L1_clk } -min -3.8 [get_ports {L1_data*  L1_enable L1_error }] -add_delay

#Output delays for 3.125Gbps Optical Links
# Use +/- 200ps skew and center clock. This assumes a setup time of 3.0ns and a hold time of 3.0ns
# The 3.125Gbps TLK3101 specs TXD setup time of 1.5ns and hold time of 0.4ns
# max_value = 3.0ns   :   min_value = -3.0ns
set_output_delay -clock { L2_clk } -max  2.5 [get_ports {L2_data*  L2_enable L2_error }]
set_output_delay -clock { L2_clk } -min -2.5 [get_ports {L2_data*  L2_enable L2_error }] -add_delay

#Output delay for ERROR signal
set_output_delay -clock { pll_125mhz } -max 1.0 [get_ports {Board_error }]
set_output_delay -clock { pll_125mhz } -min 1.0 [get_ports {Board_error }]

#Lock ref control signal for TLC3101 or TLK2501 transceiver
set_output_delay -clock { pll_125mhz } -max 4.0 [get_ports {lckrefn_l1 lckrefn_l2}]
set_output_delay -clock { pll_125mhz } -min 1.0 [get_ports {lckrefn_l1 lckrefn_l2}]

#Output delays for slow signals 
set_output_delay -clock { pll_125mhz } -max 2.0 [get_ports {*led* }]
set_output_delay -clock { pll_125mhz } -min 1.0 [get_ports {*led* }]
set_output_delay -clock { pll_125mhz }      2.0 [get_ports *chip*]

#Output delays for external clocks
set_output_delay -clock { L1_clk }        0 [get_ports {L1_clk}]
set_output_delay -clock { L2_clk }        0 [get_ports {L2_clk}]
set_output_delay -clock { samplingclock } 0 [get_ports {samplingclock}]

#Multicycle paths.  Some path are allowed upto 3 cycles to meet set time by design 
set_multicycle_path -from [get_clocks {pll_125mhz}]     -to [get_ports *led*] -setup -end 3
set_multicycle_path -from [get_clocks {pll_125mhz}]     -to [get_ports *led*] -hold  -end 3
#set_multicycle_path -from [get_registers control*_rom*] -to [get_registers control*inst25*] -setup -end 3
#set_multicycle_path -from [get_registers control*_rom*] -to [get_registers control*inst25*] -hold  -end 3
set_multicycle_path -from [get_registers control*_rom*] -to [get_registers control*inst47*] -setup -end 3
set_multicycle_path -from [get_registers control*_rom*] -to [get_registers control*inst47*] -hold  -end 3
set_multicycle_path -from [get_registers control*_rom*] -to [get_registers control*inst139*] -setup -end 3
set_multicycle_path -from [get_registers control*_rom*] -to [get_registers control*inst139*] -hold  -end 3

#Reset Multicycle path
set_multicycle_path -from [get_registers {control:inst16|ff3:inst74|lpm_ff:lpm_ff_component|dffs[0]}] -to [get_registers *adc_sync*fifo*]       -setup -end 2
set_multicycle_path -from [get_registers {control:inst16|ff3:inst74|lpm_ff:lpm_ff_component|dffs[0]}] -to [get_registers *adc_sync*fifo*]       -hold  -end 2
set_multicycle_path -from [get_registers {control:inst16|ff3:inst74|lpm_ff:lpm_ff_component|dffs[0]}] -to [get_registers data_buffer*fifo*]     -setup -end 2
set_multicycle_path -from [get_registers {control:inst16|ff3:inst74|lpm_ff:lpm_ff_component|dffs[0]}] -to [get_registers data_buffer*fifo*]     -hold  -end 2
set_multicycle_path -from [get_registers {control:inst16|ff3:inst74|lpm_ff:lpm_ff_component|dffs[0]}] -to [get_registers data_buffer*counter1*] -setup -end 2
set_multicycle_path -from [get_registers {control:inst16|ff3:inst74|lpm_ff:lpm_ff_component|dffs[0]}] -to [get_registers data_buffer*counter1*] -hold  -end 2
set_multicycle_path -from [get_registers control*sr4*inst3*\[1\]] -to [get_registers loopbac*daisy*fifo*aclr*] -setup -end 2
set_multicycle_path -from [get_registers control*sr4*inst3*\[1\]] -to [get_registers loopbac*daisy*fifo*aclr*] -hold -end 2

#False path,  LED display.
set_false_path -from [get_registers *] -to [get_ports led*]
set_false_path -from [get_ports vme*]  -to [get_ports led*]

#False path    Serdes chip enable
set_false_path -from  [get_clocks {clk_125mhz}]       -to [get_ports {L1_chip_enable L2_chip_enable}]

#False path    Test mode adc clock capture by clk_125mhz
set_false_path -from  [get_clocks *adc*]              -to [get_registers *adc_500mhz_sync_channel*ff3*inst10*]

#False Path  Test Mode for data capture
#set_false_path -from [get_registers packet*inst15*] -to [get_registers packet*fifo*q_b*]
#set_false_path -from [get_registers packet*inst15*] -to [get_registers packet*fifo*portb*]
#set_false_path -from [get_registers packet*inst15*] -to [get_registers packet*fifo*rdptr_g*]
#set_false_path -from [get_registers packet*inst15*] -to [get_registers packet*fifo*mixed_widths*]
#set_false_path -from [get_registers packet*inst26*] -to [get_registers packet*fifo*]
set_false_path -from [get_registers new_packet*L2*fifo*] -to [get_registers *vme_interface*]
set_false_path -from [get_registers new_packet*inst41*]  -to [get_registers *vme_interface*]

#Multicycle path for reset in loopback test mode
set_multicycle_path -from [get_registers loopback*inst3*]            -to [get_registers loopback*fifo*aclr*]    -setup -end 2
set_multicycle_path -from [get_registers loopback*inst3*]            -to [get_registers loopback*fifo*aclr*]    -hold  -end 2
#set_multicycle_path -from [get_registers loopback*inst13*counter16*] -to [get_registers vme_interface*inst79*] -setup -end 2
#set_multicycle_path -from [get_registers loopback*inst13*counter16*] -to [get_registers vme_interface*inst79*] -hold  -end 2

#************************************************************************************
#********VME Interface  Asynchronous Interface, timing meet by interface spec.*******
#************************************************************************************

#VME Interface to veto_trigger block Asynchronous read/write
set_false_path -from [get_registers *energy_sum*] -to [get_ports vme_data*]
set_false_path -from [get_keepers energy_sum*]    -to [get_ports vme_data*]

#VME Interface to L2 FIFO read out counterread from L2 VME Fifo 
set_false_path -from [get_registers new_packet_assembler*edge_counter*] -to [get_registers vme_interface*]
#set_false_path -from [get_registers new_packet*ff3:inst26*]             -to [get_registers packet*_fifo*qo81*]

#VME Interface between loopback_test and VME
set_false_path -from [get_registers loopback_test*] -to [get_registers vme_interface*]

#VME Interface to L2_testmode_fifo
set_multicycle_path -from [get_registers new_packet*inst15*] -to [get_registers new_packet*_fifo*]  -setup -end 2
set_multicycle_path -from [get_registers new_packet*inst15*] -to [get_registers new_packet*_fifo*]  -hold  -end 2
#set_multicycle_path -from [get_registers packet*_fifo*]  -to [get_registers packet*inst41*] -setup -end 2
#set_multicycle_path -from [get_registers packet*_fifo*]  -to [get_registers packet*inst41*] -hold  -end 2

#VME Interface    Asynchronous Interface I/O timing
create_clock -period 160 -name vme_virtual_clock
create_clock -period 160 -name vme_ds0             -waveform { 120 192 } [get_ports {vme_ds0}]
create_clock -period 160 -name vme_ds1             -waveform { 120 192 } [get_ports {vme_ds1}]
create_clock -period 160 -name vme_as              -waveform { 104 192 } [get_ports {vme_as}]
create_generated_clock   -name vme_clk             -source [get_ports {vme_ds0}]        -divide_by 1 
create_generated_clock   -name energy_sum_clk      -source [get_pins {vme_ds0|combout}] -divide_by 1 [get_registers vme_interface*inst111*]
create_generated_clock   -name data_buffer_clk     -source [get_pins {vme_ds0|combout}] -divide_by 1 [get_registers vme_interface*inst134*]


set_input_delay  -clock { vme_virtual_clock } -max 5 [get_ports {vme_address* vme_am* vme_ga* vme_iackin vme_lword vme_write vme_data* vme_ds*}]
set_input_delay  -clock { vme_virtual_clock } -min 2 [get_ports {vme_address* vme_am* vme_ga* vme_iackin vme_lword vme_write vme_data* vme_ds*}]
set_output_delay -clock { pll_125mhz }        -max 2 [get_ports {vme_dir vme_dtack }]
set_output_delay -clock { pll_125mhz }        -min 1 [get_ports {vme_dir vme_dtack }]
set_output_delay -clock { pll_125mhz }        -max 2 [get_ports {vme_data* }]
set_output_delay -clock { pll_125mhz }        -min 1 [get_ports {vme_data* }]

#VME register and port timing within vme block  allow 3 cycles
set_multicycle_path -from [get_registers vme_interface*]   -to [get_registers vme_interface*]      -setup 2
set_multicycle_path -from [get_registers vme_interface*]   -to [get_registers vme_interface*]      -hold  2
set_multicycle_path -from [get_registers vme_interface*]   -to [get_ports {vme_* }]                -setup 3
set_multicycle_path -from [get_registers vme_interface*]   -to [get_ports {vme_* }]                -hold  3
set_multicycle_path -from [get_registers vme_interface*]   -to [get_ports {vme_data* vme_dtack}]   -setup 4
set_multicycle_path -from [get_registers vme_interface*]   -to [get_ports {vme_data* vme_dtack}]   -hold  4
set_multicycle_path -from [get_registers vme_interface*]   -to [get_ports {lckrefn_l1 lckrefn_l2}] -setup 4
set_multicycle_path -from [get_registers vme_interface*]   -to [get_ports {lckrefn_l1 lckrefn_l2}] -hold  4
set_multicycle_path -from [get_ports {vme_ds* }]           -to [get_ports {vme_data* vme_dir}]          -setup 3
set_multicycle_path -from [get_ports {vme_ds* }]           -to [get_ports {vme_data* vme_dir}]          -hold  3
set_multicycle_path -from [get_ports {vme_ds* }]           -to [get_registers vme_interface*]           -setup 2
set_multicycle_path -from [get_ports {vme_ds* }]           -to [get_registers vme_interface*]           -hold  2
set_multicycle_path -from [get_ports vme_data*]            -to [get_registers vme_interface*reg*]       -setup 3
set_multicycle_path -from [get_ports vme_data*]            -to [get_registers vme_interface*reg*]       -hold  3
 

#************************************************************************************
#*******                 VME timing with external blocks                     ********
#*******      All vme control registers are to be up dated when LIVE=0       ********
#*******      Status registers that are monitored when LIVE=1 should         ********
#*******      be read multiple time to eliminate asynchronous timing.        ********
#************************************************************************************

#VME  register timing between vme block allow 3 cycles
set_multicycle_path -from [get_registers vme_interface*]  -to [get_registers *]  -setup 3
set_multicycle_path -from [get_registers vme_interface*]  -to [get_registers *]  -hold  3
set_multicycle_path -from [get_registers *]  -to [get_registers vme*inst101*]    -setup 3
set_multicycle_path -from [get_registers *]  -to [get_registers vme*inst101*]    -hold  3

#VME   Serdes chip enable signals
set_multicycle_path -from [get_registers *vme_interface*vme_reg16*inst4*[12]*] -to [get_ports {*chip_enable* }] -setup 3
set_multicycle_path -from [get_registers *vme_interface*vme_reg16*inst4*[12]*] -to [get_ports {*chip_enable* }] -hold  3
#VME Tied internally 
set_false_path -from [get_ports {vme_iackin}]               -to [get_ports {vme_iackout}]

# Asynchronous signal Board_error
set_false_path -from [get_registers *] -to [get_ports Board*]

# Asynchronous signal read_crc_error
set_false_path -from [get_ports {read_crc_error}] -to [get_registers {vme_interface*}]

#Design Assistance  warnings suppressed
#
#  D101  vme address    ( vme bus timing prevents this condition )
#  D103  vme address    ( vme bus timing prevents this condition )
#  R104  vme address    ( vme bus timing prevents this condition )
#  C104  vme_clk        ( done intentionally for asynchronous bus )
#  C104  adc_clk's      ( test mode for adc_clk alignment )
#  D102  adc_clk and start0 alignment issue ( need to use samplingclock for correct alignment )
#  D103  start0 and adc_clk's  ( need to use samplingclock for correct alignment )
#  D102  fifo testmode  ( vme control signal prevents this condition )


